/*
   libflame
   An object-based infrastructure for developing high-performance
   dense linear algebra libraries.

   Copyright (C) 2009, The University of Texas

   libflame is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as
   published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   libflame is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with libflame; if you did not receive a copy, see
   http://www.gnu.org/licenses/.

   For more information, please contact us at flame@cs.utexas.edu or
   send mail to:

   Field G. Van Zee and/or
   Robert A. van de Geijn
   The University of Texas at Austin
   Department of Computer Sciences
   1 University Station C0500
   Austin TX 78712
*/

// Define a Fortran integer type
#define F_INT int

// Define a Fortran double precision floating point type
#define F_DOUBLE double

// Windows has weird underscoring. Handle this separately.
#ifdef FLA_ENABLE_WINDOWS_BUILD

  #define FLA_C2F( name ) name
  #define FLA_F2C( name ) name

#else // Linux case

  // Handle the various scenarios for Fortran underscoring with respect to the
  // FLA_C2F and FLA_F2C macros.
  #ifdef FLA_F77_UNDERSCORE
    #ifdef FLA_F77_EXTRA_UNDERSCORE

      // Case 1: configure detected that the Fortran 77 compiler employs both
      // an underscore for Fortran symbols as well as an extra underscore in
      // symbol names that already contain at least one underscore.
      #define FLA_C2F( name ) name ## _
      #define FLA_F2C( name ) name ## __

    #else

      // Case 2: configure detected that the Fortran 77 compiler employs only
      // an underscore for Fortran symbols (and no additional underscores).
      #define FLA_C2F( name ) name ## _
      #define FLA_F2C( name ) name ## _

    #endif
  #else
    #ifdef FLA_F77_EXTRA_UNDERSCORE

      // Case 3: configure detected that the Fortran 77 compiler employs only
      // an extra underscore for Fortran symbols that already contain at least
      // least one underscore. This case seems awkward, but it has been placed
      // here anyway for completeness. The macros below may not be correct
      // depending on the meaning of the "no underscore, extra underscore"
      // result from configure.
      #define FLA_C2F( name ) name
      #define FLA_F2C( name ) name ## __

    #else

      // Case 4: configure detected that the Fortran 77 compiler employs neither
      // an underscore for Fortran symbols nor an extra underscore in symbol
      // names that already contain at least one underscore. Another strange
      // scenario, and so the macros below are configured somewhat speculatively.
      #define FLA_C2F( name ) name
      #define FLA_F2C( name ) name

    #endif
  #endif
#endif
