/*
   libflame
   An object-based infrastructure for developing high-performance
   dense linear algebra libraries.

   Copyright (C) 2009, The University of Texas

   libflame is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as
   published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   libflame is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with libflame; if you did not receive a copy, see
   http://www.gnu.org/licenses/.

   For more information, please contact us at flame@cs.utexas.edu or
   send mail to:

   Field G. Van Zee and/or
   Robert A. van de Geijn
   The University of Texas at Austin
   Department of Computer Sciences
   1 University Station C0500
   Austin TX 78712
*/

#include "FLA_F2C.h"

// --- single-precision real level-1 BLAS ------------------------------------------------
float    FLA_C2F( sasum  )( int* n, float* x, int* incx );
void     FLA_C2F( saxpy  )( int* n, float* alpha, float* x, int* incx,  float* y, int* incy );
void     FLA_C2F( scopy  )( int* n, float* x, int* incx, float* y, int* incy );
float    FLA_C2F( sdot   )( int* n, float* x, int* incx, float* y, int* incy );
int      FLA_C2F( isamax )( int* n, float* x, int* incx );
float    FLA_C2F( snrm2  )( int* n, float* x, int* incx );
void     FLA_C2F( sscal  )( int* n, float* alpha, float* y, int* incy );
void     FLA_C2F( sswap  )( int* n, float* x, int* incx, float* y, int* incy );

// --- double-precision real level-1 BLAS ------------------------------------------------
double   FLA_C2F( dasum  )( int* n, double* x, int* incx );
void     FLA_C2F( daxpy  )( int* n, double* alpha, double* x, int* incx,  double* y, int* incy );
void     FLA_C2F( dcopy  )( int* n, double* x, int* incx, double* y, int* incy );
double   FLA_C2F( ddot   )( int* n, double* x, int* incx, double* y, int* incy );
int      FLA_C2F( idamax )( int* n, double* x, int* incx );
double   FLA_C2F( dnrm2  )( int* n, double* x, int* incx );
void     FLA_C2F( dscal  )( int* n, double* alpha, double* y, int* incy );
void     FLA_C2F( dswap  )( int* n, double* x, int* incx, double* y, int* incy );

// --- single-precision complex level-1 BLAS ---------------------------------------------
float    FLA_C2F( scasum )( int* n, scomplex* x, int* incx );
void     FLA_C2F( caxpy  )( int* n, scomplex* alpha, scomplex* x, int* incx,  scomplex* y, int* incy );
void     FLA_C2F( ccopy  )( int* n, scomplex* x, int* incx, scomplex* y, int* incy );
scomplex FLA_C2F( cdotu  )( int* n, scomplex* x, int* incx, scomplex* y, int* incy );
scomplex FLA_C2F( cdotc  )( int* n, scomplex* x, int* incx, scomplex* y, int* incy );
int      FLA_C2F( icamax )( int* n, scomplex* x, int* incx );
float    FLA_C2F( scnrm2 )( int* n, scomplex* x, int* incx );
void     FLA_C2F( cscal  )( int* n, scomplex* alpha, scomplex* y, int* incy );
void     FLA_C2F( csscal )( int* n, float*   alpha, scomplex* y, int* incy );
void     FLA_C2F( cswap  )( int* n, scomplex* x, int* incx, scomplex* y, int* incy );

// --- double-precision complex level-1 BLAS ---------------------------------------------
double   FLA_C2F( dzasum )( int* n, dcomplex* x, int* incx );
void     FLA_C2F( zaxpy  )( int* n, dcomplex* alpha, dcomplex* x, int* incx,  dcomplex* y, int* incy );
void     FLA_C2F( zcopy  )( int* n, dcomplex* x, int* incx, dcomplex* y, int* incy );
dcomplex FLA_C2F( zdotu  )( int* n, dcomplex* x, int* incx, dcomplex* y, int* incy );
dcomplex FLA_C2F( zdotc  )( int* n, dcomplex* x, int* incx, dcomplex* y, int* incy );
int      FLA_C2F( izamax )( int* n, dcomplex* x, int* incx );
double   FLA_C2F( dznrm2 )( int* n, dcomplex* x, int* incx );
void     FLA_C2F( zscal  )( int* n, dcomplex* alpha, dcomplex* y, int* incy );
void     FLA_C2F( zdscal )( int* n, double* alpha, dcomplex* y, int* incy );
void     FLA_C2F( zswap  )( int* n, dcomplex* x, int* incx, dcomplex* y, int* incy );



// --- single-precision real level-2 BLAS ------------------------------------------------
void     FLA_C2F( sgemv  )( char* transa, int* m, int* n, float* alpha, float *a, int* lda, float* x, int* incx, float* beta, float *y, int* incy );
void     FLA_C2F( sger   )( int* m, int* n, float* alpha, float* x, int* incx, float* y, int* incy, float* a, int* lda );
void     FLA_C2F( ssymv  )( char* uplo, int* n, float* alpha, float* a, int* lda, float* x, int* incx, float* beta, float* y, int* incy );
void     FLA_C2F( ssyr   )( char* uplo, int* n, float* alpha, float* x, int* incx, float* a, int* lda );
void     FLA_C2F( ssyr2  )( char* uplo, int* n, float* alpha, float* x, int* incx, float* y, int* incy, float* a, int* lda );
void     FLA_C2F( strmv  )( char* uplo, char* transa, char* diag, int* n,  float* a, int* lda, float* y, int* incy );
void     FLA_C2F( strsv  )( char* uplo, char* transa, char* diag, int* n,  float* a, int* lda, float* y, int* incy );

// --- double-precision real level-2 BLAS ------------------------------------------------
void     FLA_C2F( dgemv  )( char* transa, int* m, int* n, double* alpha, double *a, int* lda, double* x, int* incx, double* beta, double *y, int* incy );
void     FLA_C2F( dger   )( int* m, int* n, double* alpha, double* x, int* incx, double* y, int* incy, double* a, int* lda );
void     FLA_C2F( dsymv  )( char* uplo, int* n, double* alpha, double* a, int* lda, double* x, int* incx, double* beta, double* y, int* incy );
void     FLA_C2F( dsyr   )( char* uplo, int* n, double* alpha, double* x, int* incx, double* a, int* lda );
void     FLA_C2F( dsyr2  )( char* uplo, int* n, double* alpha, double* x, int* incx, double* y, int* incy, double* a, int* lda );
void     FLA_C2F( dtrmv  )( char* uplo, char* transa, char* diag, int* n,  double* a, int* lda, double* y, int* incy );
void     FLA_C2F( dtrsv  )( char* uplo, char* transa, char* diag, int* n,  double* a, int* lda, double* y, int* incy );

// --- single-precision complex level-2 BLAS ---------------------------------------------
void     FLA_C2F( cgemv  )( char* transa, int* m, int* n, scomplex* alpha, scomplex *a, int* lda, scomplex* x, int* incx, scomplex* beta, scomplex *y, int* incy );
void     FLA_C2F( cgerc  )( int* m, int* n, scomplex* alpha, scomplex* x, int* incx, scomplex* y, int* incy, scomplex* a, int* lda );
void     FLA_C2F( cgeru  )( int* m, int* n, scomplex* alpha, scomplex* x, int* incx, scomplex* y, int* incy, scomplex* a, int* lda );
void     FLA_C2F( chemv  )( char* uplo, int* n, scomplex* alpha, scomplex* a, int* lda, scomplex* x, int* incx, scomplex* beta, scomplex* y, int* incy );
void     FLA_C2F( cher   )( char* uplo, int* n, float* alpha, scomplex* x, int* incx, scomplex* a, int* lda );
void     FLA_C2F( cher2  )( char* uplo, int* n, scomplex* alpha, scomplex* x, int* incx, scomplex* y, int* incy, scomplex* a, int* lda );
void     FLA_C2F( ctrmv  )( char* uplo, char* transa, char* diag, int* n,  scomplex* a, int* lda, scomplex* y, int* incy );
void     FLA_C2F( ctrsv  )( char* uplo, char* transa, char* diag, int* n,  scomplex* a, int* lda, scomplex* y, int* incy );

// --- double-precision complex level-2 BLAS ---------------------------------------------
void     FLA_C2F( zgemv  )( char* transa, int* m, int* n, dcomplex* alpha, dcomplex *a, int* lda, dcomplex* x, int* incx, dcomplex* beta, dcomplex *y, int* incy );
void     FLA_C2F( zgerc  )( int* m, int* n, dcomplex* alpha, dcomplex* x, int* incx, dcomplex* y, int* incy, dcomplex* a, int* lda );
void     FLA_C2F( zgeru  )( int* m, int* n, dcomplex* alpha, dcomplex* x, int* incx, dcomplex* y, int* incy, dcomplex* a, int* lda );
void     FLA_C2F( zhemv  )( char* uplo, int* n, dcomplex* alpha, dcomplex* a, int* lda, dcomplex* x, int* incx, dcomplex* beta, dcomplex* y, int* incy );
void     FLA_C2F( zher   )( char* uplo, int* n, double* alpha, dcomplex* x, int* incx, dcomplex* a, int* lda );
void     FLA_C2F( zher2  )( char* uplo, int* n, dcomplex* alpha, dcomplex* x, int* incx, dcomplex* y, int* incy, dcomplex* a, int* lda );
void     FLA_C2F( ztrmv  )( char* uplo, char* transa, char* diag, int* n,  dcomplex* a, int* lda, dcomplex* y, int* incy );
void     FLA_C2F( ztrsv  )( char* uplo, char* transa, char* diag, int* n,  dcomplex* a, int* lda, dcomplex* y, int* incy );



// --- single-precision real level-3 BLAS ------------------------------------------------
void     FLA_C2F( sgemm  )( char* transa, char* transb, int* m, int* n, int* k, float* alpha, float* a, int* lda, float* b, int* ldb, float* beta, float* c, int* ldc );
void     FLA_C2F( ssymm  )( char* side, char* uplo, int* m, int* n, float* alpha, float* a, int* lda, float* b, int* ldb, float* beta, float* c, int* ldc );
void     FLA_C2F( ssyrk  )( char* uplo, char* transa, int* n, int* k, float* alpha, float* a, int* lda, float* beta, float* c, int* ldc );
void     FLA_C2F( ssyr2k )( char* uplo, char* transa, int* n, int* k, float* alpha, float* a, int* lda, float* b, int* ldb, float* beta, float* c, int* ldc );
void     FLA_C2F( strmm  )( char* side, char* uplo, char* transa, char* diag, int* m, int* n, float* alpha, float* a, int* lda, float* b, int* ldb );
void     FLA_C2F( strsm  )( char* side, char* uplo, char* transa, char* diag, int* m, int* n, float* alpha, float* a, int* lda, float* b, int* ldb );

// --- double-precision real level-3 BLAS ------------------------------------------------
void     FLA_C2F( dgemm  )( char* transa, char* transb, int* m, int* n, int* k, double* alpha, double* a, int* lda, double* b, int* ldb, double* beta, double* c, int* ldc );
void     FLA_C2F( dsymm  )( char* side, char* uplo, int* m, int* n, double* alpha, double* a, int* lda, double* b, int* ldb, double* beta, double* c, int* ldc );
void     FLA_C2F( dsyrk  )( char* uplo, char* transa, int* n, int* k, double* alpha, double* a, int* lda, double* beta, double* c, int* ldc );
void     FLA_C2F( dsyr2k )( char* uplo, char* transa, int* n, int* k, double* alpha, double* a, int* lda, double* b, int* ldb, double* beta, double* c, int* ldc );
void     FLA_C2F( dtrmm  )( char* side, char* uplo, char* transa, char* diag, int* m, int* n, double* alpha, double* a, int* lda, double* b, int* ldb );
void     FLA_C2F( dtrsm  )( char* side, char* uplo, char* transa, char* diag, int* m, int* n, double* alpha, double* a, int* lda, double* b, int* ldb );

// --- single-precision complex level-3 BLAS ---------------------------------------------
void     FLA_C2F( cgemm  )( char* transa, char* transb, int* m, int* n, int* k, scomplex* alpha, scomplex* a, int* lda, scomplex* b, int* ldb, scomplex* beta, scomplex* c, int* ldc );
void     FLA_C2F( chemm  )( char* side, char* uplo, int* m, int* n, scomplex* alpha, scomplex* a, int* lda, scomplex* b, int* ldb, scomplex* beta, scomplex* c, int* ldc );
void     FLA_C2F( cherk  )( char* uplo, char* transa, int* n, int* k, float* alpha, scomplex* a, int* lda, float* beta, scomplex* c, int* ldc );
void     FLA_C2F( cher2k )( char* uplo, char* transa, int* n, int* k, scomplex* alpha, scomplex* a, int* lda, scomplex* b, int* ldb, float* beta, scomplex *c, int* ldc );
void     FLA_C2F( csymm  )( char* side, char* uplo, int* m, int* n, scomplex* alpha, scomplex* a, int* lda, scomplex* b, int* ldb, scomplex* beta, scomplex* c, int* ldc );
void     FLA_C2F( csyrk  )( char* uplo, char* transa, int* n, int* k, scomplex* alpha, scomplex* a, int* lda, scomplex* beta, scomplex* c, int* ldc );
void     FLA_C2F( csyr2k )( char* uplo, char* transa, int* n, int* k, scomplex* alpha, scomplex* a, int* lda, scomplex* b, int* ldb, scomplex* beta, scomplex* c, int* ldc );
void     FLA_C2F( ctrmm  )( char* side, char* uplo, char* transa, char* diag, int* m, int* n, scomplex* alpha, scomplex* a, int* lda, scomplex* b, int* ldb );
void     FLA_C2F( ctrsm  )( char* side, char* uplo, char* transa, char* diag, int* m, int* n, scomplex* alpha, scomplex* a, int* lda, scomplex* b, int* ldb );

// --- double-precision complex level-3 BLAS ---------------------------------------------
void     FLA_C2F( zgemm  )( char* transa, char* transb, int* m, int* n, int* k, dcomplex* alpha, dcomplex* a, int* lda, dcomplex* b, int* ldb, dcomplex* beta, dcomplex* c, int* ldc );
void     FLA_C2F( zhemm  )( char* side, char* uplo, int* m, int* n, dcomplex* alpha, dcomplex* a, int* lda, dcomplex* b, int* ldb, dcomplex* beta, dcomplex* c, int* ldc );
void     FLA_C2F( zherk  )( char* uplo, char* transa, int* n, int* k, double* alpha, dcomplex* a, int* lda, double* beta, dcomplex* c, int* ldc );
void     FLA_C2F( zher2k )( char* uplo, char* transa, int* n, int* k, dcomplex* alpha, dcomplex* a, int* lda, dcomplex* b, int* ldb, double* beta, dcomplex *c, int* ldc );
void     FLA_C2F( zsymm  )( char* side, char* uplo, int* m, int* n, dcomplex* alpha, dcomplex* a, int* lda, dcomplex* b, int* ldb, dcomplex* beta, dcomplex* c, int* ldc );
void     FLA_C2F( zsyrk  )( char* uplo, char* transa, int* n, int* k, dcomplex* alpha, dcomplex* a, int* lda, dcomplex* beta, dcomplex* c, int* ldc );
void     FLA_C2F( zsyr2k )( char* uplo, char* transa, int* n, int* k, dcomplex* alpha, dcomplex* a, int* lda, dcomplex* b, int* ldb, dcomplex* beta, dcomplex* c, int* ldc );
void     FLA_C2F( ztrmm  )( char* side, char* uplo, char* transa, char* diag, int* m, int* n, dcomplex* alpha, dcomplex* a, int* lda, dcomplex* b, int* ldb );
void     FLA_C2F( ztrsm  )( char* side, char* uplo, char* transa, char* diag, int* m, int* n, dcomplex* alpha, dcomplex* a, int* lda, dcomplex* b, int* ldb );

