/*
   libflame
   An object-based infrastructure for developing high-performance
   dense linear algebra libraries.

   Copyright (C) 2009, The University of Texas

   libflame is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as
   published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   libflame is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with libflame; if you did not receive a copy, see
   http://www.gnu.org/licenses/.

   For more information, please contact us at flame@cs.utexas.edu or
   send mail to:

   Field G. Van Zee and/or
   Robert A. van de Geijn
   The University of Texas at Austin
   Department of Computer Sciences
   1 University Station C0500
   Austin TX 78712
*/

#include "FLA_F2C.h"

// --- single-precision real LAPACK ------------------------------------------------------
void FLA_C2F( spotrf )( char* uplo, int* n, float* a, int* lda, int* info );
void FLA_C2F( spotf2 )( char* uplo, int* n, float* a, int* lda, int* info );

void FLA_C2F( sgetrf )( int* m, int* n, float* a, int* lda, int* ipiv, int* info );
void FLA_C2F( sgetf2 )( int* m, int* n, float* a, int* lda, int* ipiv, int* info );

void FLA_C2F( sgetnf )( int* m, int* n, float* a, int* lda, int* info );
void FLA_C2F( sgetn2 )( int* m, int* n, float* a, int* lda, int* info );

void FLA_C2F( sgeqrf )( int* m, int* n, float* a, int* lda, float* tau, float* work, int* lwork, int* info );
void FLA_C2F( sgeqr2 )( int* m, int* n, float* a, int* lda, float* tau, float* work, int* info );

void FLA_C2F( sgeqrfut )( int* m, int* n, float* a, int* lda, float* tau, float* work, int* lwork, int* info );
void FLA_C2F( sgeqr2ut )( int* m, int* n, float* a, int* lda, float* tau, float* work, int* info );

void FLA_C2F( sgelqf )( int* m, int* n, float* a, int* lda, float* tau, float* work, int* lwork, int* info );
void FLA_C2F( sgelq2 )( int* m, int* n, float* a, int* lda, float* tau, float* work, int* info );

void FLA_C2F( sgelqfut )( int* m, int* n, float* a, int* lda, float* tau, float* work, int* lwork, int* info );
void FLA_C2F( sgelq2ut )( int* m, int* n, float* a, int* lda, float* tau, float* work, int* info );

void FLA_C2F( slarftut )( char* direct, char* storev, int* n, int* k, float* v, int* ldv, float* tau, float* t, int* ldt );
void FLA_C2F( slarfbut )( char* side, char* trans, char* direct, char* storev, int* m, int* n, int* k, float* v, int* ldv, float* t, int* ldt, float* c, int* ldc, float* work, int* ldwork );
void FLA_C2F( slarfgut )( int* n, float* alpha, float* x, int* incx, float* tau );

void FLA_C2F( sgehrd )( int* n, int* ilo, int* ihi, float* a, int* lda, float* tau, float* work, int* lwork, int* info );
void FLA_C2F( sgehd2 )( int* n, int* ilo, int* ihi, float* a, int* lda, float* tau, float* work, int* info );

void FLA_C2F( strtri )( char* uplo, char* diag, int* n, float* a, int* lda, int* info );
void FLA_C2F( strti2 )( char* uplo, char* diag, int* n, float* a, int* lda, int* info );

void FLA_C2F( slauum )( char* uplo, int* n, float* a, int* lda, int* info );
void FLA_C2F( slauu2 )( char* uplo, int* n, float* a, int* lda, int* info );

void FLA_C2F( strsyl )( char* transa, char* transb, int* isgn, int* m, int* n, float* a, int* lda, float* b, int* ldb, float* c, int* ldc, float* scale, int* info );

void FLA_C2F( slaswp )( int* n, float* a, int* lda, int* k1, int* k2, int* ipiv, int* incx );
void FLA_C2F( slarft )( char* direct, char* storev, int* n, int* k, float* v, int* ldv, float* tau, float* t, int* ldt );
void FLA_C2F( slarfg )( int* n, float* alpha, float* x, int* incx, float* tau );
void FLA_C2F( slaset )( char* uplo, int* m, int* n, float* alpha, float* beta, float* a, int* lda );

void FLA_C2F( sormqr )( char* side, char* trans, int* m, int* n, int* k, float* a, int* lda, float* tau, float* c, int* ldc, float* work, int* lwork, int* info );
void FLA_C2F( sorm2r )( char* side, char* trans, int* m, int* n, int* k, float* a, int* lda, float* tau, float* c, int* ldc, float* work, int* info );
void FLA_C2F( sormlq )( char* side, char* trans, int* m, int* n, int* k, float* a, int* lda, float* tau, float* c, int* ldc, float* work, int* lwork, int* info );
void FLA_C2F( sorml2 )( char* side, char* trans, int* m, int* n, int* k, float* a, int* lda, float* tau, float* c, int* ldc, float* work, int* info );

// --- double-precision real LAPACK ------------------------------------------------------
void FLA_C2F( dpotrf )( char* uplo, int* n, double* a, int* lda, int* info );
void FLA_C2F( dpotf2 )( char* uplo, int* n, double* a, int* lda, int* info );

void FLA_C2F( dgetrf )( int* m, int* n, double* a, int* lda, int* ipiv, int* info );
void FLA_C2F( dgetf2 )( int* m, int* n, double* a, int* lda, int* ipiv, int* info );

void FLA_C2F( dgetnf )( int* m, int* n, double* a, int* lda, int* info );
void FLA_C2F( dgetn2 )( int* m, int* n, double* a, int* lda, int* info );

void FLA_C2F( dgeqrf )( int* m, int* n, double* a, int* lda, double* tau, double* work, int* lwork, int* info );
void FLA_C2F( dgeqr2 )( int* m, int* n, double* a, int* lda, double* tau, double* work, int* info );

void FLA_C2F( dgeqrfut )( int* m, int* n, double* a, int* lda, double* tau, double* work, int* lwork, int* info );
void FLA_C2F( dgeqr2ut )( int* m, int* n, double* a, int* lda, double* tau, double* work, int* info );

void FLA_C2F( dgelqf )( int* m, int* n, double* a, int* lda, double* tau, double* work, int* lwork, int* info );
void FLA_C2F( dgelq2 )( int* m, int* n, double* a, int* lda, double* tau, double* work, int* info );

void FLA_C2F( dgelqfut )( int* m, int* n, double* a, int* lda, double* tau, double* work, int* lwork, int* info );
void FLA_C2F( dgelq2ut )( int* m, int* n, double* a, int* lda, double* tau, double* work, int* info );

void FLA_C2F( dlarftut )( char* direct, char* storev, int* n, int* k, double* v, int* ldv, double* tau, double* t, int* ldt );
void FLA_C2F( dlarfbut )( char* side, char* trans, char* direct, char* storev, int* m, int* n, int* k, double* v, int* ldv, double* t, int* ldt, double* c, int* ldc, double* work, int* ldwork );
void FLA_C2F( dlarfgut )( int* n, double* alpha, double* x, int* incx, double* tau );

void FLA_C2F( dgehrd )( int* n, int* ilo, int* ihi, double* a, int* lda, double* tau, double* work, int* lwork, int* info );
void FLA_C2F( dgehd2 )( int* n, int* ilo, int* ihi, double* a, int* lda, double* tau, double* work, int* info );

void FLA_C2F( dtrtri )( char* uplo, char* diag, int* n, double* a, int* lda, int* info );
void FLA_C2F( dtrti2 )( char* uplo, char* diag, int* n, double* a, int* lda, int* info );

void FLA_C2F( dlauum )( char* uplo, int* n, double* a, int* lda, int* info );
void FLA_C2F( dlauu2 )( char* uplo, int* n, double* a, int* lda, int* info );

void FLA_C2F( dtrsyl )( char* transa, char* transb, int* isgn, int* m, int* n, double* a, int* lda, double* b, int* ldb, double* c, int* ldc, double* scale, int* info );

void FLA_C2F( dlaswp )( int* n, double* a, int* lda, int* k1, int* k2, int* ipiv, int* incx );
void FLA_C2F( dlarft )( char* direct, char* storev, int* n, int* k, double* v, int* ldv, double* tau, double* t, int* ldt );
void FLA_C2F( dlarfg )( int* n, double* alpha, double* x, int* incx, double* tau );
void FLA_C2F( dlaset )( char* uplo, int* m, int* n, double* alpha, double* beta, double* a, int* lda );

void FLA_C2F( dormqr )( char* side, char* trans, int* m, int* n, int* k, double* a, int* lda, double* tau, double* c, int* ldc, double* work, int* lwork, int* info );
void FLA_C2F( dorm2r )( char* side, char* trans, int* m, int* n, int* k, double* a, int* lda, double* tau, double* c, int* ldc, double* work, int* info );
void FLA_C2F( dormlq )( char* side, char* trans, int* m, int* n, int* k, double* a, int* lda, double* tau, double* c, int* ldc, double* work, int* lwork, int* info );
void FLA_C2F( dorml2 )( char* side, char* trans, int* m, int* n, int* k, double* a, int* lda, double* tau, double* c, int* ldc, double* work, int* info );

// --- single-precision complex LAPACK ---------------------------------------------------
void FLA_C2F( cpotrf )( char* uplo, int* n, scomplex* a, int* lda, int* info );
void FLA_C2F( cpotf2 )( char* uplo, int* n, scomplex* a, int* lda, int* info );

void FLA_C2F( cgetrf )( int* m, int* n, scomplex* a, int* lda, int* ipiv, int* info );
void FLA_C2F( cgetf2 )( int* m, int* n, scomplex* a, int* lda, int* ipiv, int* info );

void FLA_C2F( cgetnf )( int* m, int* n, scomplex* a, int* lda, int* info );
void FLA_C2F( cgetn2 )( int* m, int* n, scomplex* a, int* lda, int* info );

void FLA_C2F( cgeqrf )( int* m, int* n, scomplex* a, int* lda, scomplex* tau, scomplex* work, int* lwork, int* info );
void FLA_C2F( cgeqr2 )( int* m, int* n, scomplex* a, int* lda, scomplex* tau, scomplex* work, int* info );

void FLA_C2F( cgeqrfut )( int* m, int* n, scomplex* a, int* lda, scomplex* tau, scomplex* work, int* lwork, int* info );
void FLA_C2F( cgeqr2ut )( int* m, int* n, scomplex* a, int* lda, scomplex* tau, scomplex* work, int* info );

void FLA_C2F( cgelqf )( int* m, int* n, scomplex* a, int* lda, scomplex* tau, scomplex* work, int* lwork, int* info );
void FLA_C2F( cgelq2 )( int* m, int* n, scomplex* a, int* lda, scomplex* tau, scomplex* work, int* info );

void FLA_C2F( cgelqfut )( int* m, int* n, scomplex* a, int* lda, scomplex* tau, scomplex* work, int* lwork, int* info );
void FLA_C2F( cgelq2ut )( int* m, int* n, scomplex* a, int* lda, scomplex* tau, scomplex* work, int* info );

void FLA_C2F( clarftut )( char* direct, char* storev, int* n, int* k, scomplex* v, int* ldv, scomplex* tau, scomplex* t, int* ldt );
void FLA_C2F( clarfbut )( char* side, char* trans, char* direct, char* storev, int* m, int* n, int* k, scomplex* v, int* ldv, scomplex* t, int* ldt, scomplex* c, int* ldc, scomplex* work, int* ldwork );
void FLA_C2F( clarfgut )( int* n, scomplex* alpha, scomplex* x, int* incx, scomplex* tau );

void FLA_C2F( cgehrd )( int* n, int* ilo, int* ihi, scomplex* a, int* lda, scomplex* tau, scomplex* work, int* lwork, int* info );
void FLA_C2F( cgehd2 )( int* n, int* ilo, int* ihi, scomplex* a, int* lda, scomplex* tau, scomplex* work, int* info );

void FLA_C2F( ctrtri )( char* uplo, char* diag, int* n, scomplex* a, int* lda, int* info );
void FLA_C2F( ctrti2 )( char* uplo, char* diag, int* n, scomplex* a, int* lda, int* info );

void FLA_C2F( clauum )( char* uplo, int* n, scomplex* a, int* lda, int* info );
void FLA_C2F( clauu2 )( char* uplo, int* n, scomplex* a, int* lda, int* info );

void FLA_C2F( ctrsyl )( char* transa, char* transb, int* isgn, int* m, int* n, scomplex* a, int* lda, scomplex* b, int* ldb, scomplex* c, int* ldc, float* scale, int* info );

void FLA_C2F( claswp )( int* n, scomplex* a, int* lda, int* k1, int* k2, int* ipiv, int* incx );
void FLA_C2F( clarft )( char* direct, char* storev, int* n, int* k, scomplex* v, int* ldv, scomplex* tau, scomplex* t, int* ldt );
void FLA_C2F( clarfg )( int* n, scomplex* alpha, scomplex* x, int* incx, scomplex* tau );
void FLA_C2F( claset )( char* uplo, int* m, int* n, scomplex* alpha, scomplex* beta, scomplex* a, int* lda );

void FLA_C2F( cunmqr )( char* side, char* trans, int* m, int* n, int* k, scomplex* a, int* lda, scomplex* tau, scomplex* c, int* ldc, scomplex* work, int* lwork, int* info );
void FLA_C2F( cunm2r )( char* side, char* trans, int* m, int* n, int* k, scomplex* a, int* lda, scomplex* tau, scomplex* c, int* ldc, scomplex* work, int* info );
void FLA_C2F( cunmlq )( char* side, char* trans, int* m, int* n, int* k, scomplex* a, int* lda, scomplex* tau, scomplex* c, int* ldc, scomplex* work, int* lwork, int* info );
void FLA_C2F( cunml2 )( char* side, char* trans, int* m, int* n, int* k, scomplex* a, int* lda, scomplex* tau, scomplex* c, int* ldc, scomplex* work, int* info );

// --- double-precision complex LAPACK ---------------------------------------------------
void FLA_C2F( zpotrf )( char* uplo, int* n, dcomplex* a, int* lda, int* info );
void FLA_C2F( zpotf2 )( char* uplo, int* n, dcomplex* a, int* lda, int* info );

void FLA_C2F( zgetrf )( int* m, int* n, dcomplex* a, int* lda, int* ipiv, int* info );
void FLA_C2F( zgetf2 )( int* m, int* n, dcomplex* a, int* lda, int* ipiv, int* info );

void FLA_C2F( zgetnf )( int* m, int* n, dcomplex* a, int* lda, int* info );
void FLA_C2F( zgetn2 )( int* m, int* n, dcomplex* a, int* lda, int* info );

void FLA_C2F( zgeqrf )( int* m, int* n, dcomplex* a, int* lda, dcomplex* tau, dcomplex* work, int* lwork, int* info );
void FLA_C2F( zgeqr2 )( int* m, int* n, dcomplex* a, int* lda, dcomplex* tau, dcomplex* work, int* info );

void FLA_C2F( zgeqrfut )( int* m, int* n, dcomplex* a, int* lda, dcomplex* tau, dcomplex* work, int* lwork, int* info );
void FLA_C2F( zgeqr2ut )( int* m, int* n, dcomplex* a, int* lda, dcomplex* tau, dcomplex* work, int* info );

void FLA_C2F( zgelqf )( int* m, int* n, dcomplex* a, int* lda, dcomplex* tau, dcomplex* work, int* lwork, int* info );
void FLA_C2F( zgelq2 )( int* m, int* n, dcomplex* a, int* lda, dcomplex* tau, dcomplex* work, int* info );

void FLA_C2F( zgelqfut )( int* m, int* n, dcomplex* a, int* lda, dcomplex* tau, dcomplex* work, int* lwork, int* info );
void FLA_C2F( zgelq2ut )( int* m, int* n, dcomplex* a, int* lda, dcomplex* tau, dcomplex* work, int* info );

void FLA_C2F( zlarftut )( char* direct, char* storev, int* n, int* k, dcomplex* v, int* ldv, dcomplex* tau, dcomplex* t, int* ldt );
void FLA_C2F( zlarfbut )( char* side, char* trans, char* direct, char* storev, int* m, int* n, int* k, dcomplex* v, int* ldv, dcomplex* t, int* ldt, dcomplex* c, int* ldc, dcomplex* work, int* ldwork );
void FLA_C2F( zlarfgut )( int* n, dcomplex* alpha, dcomplex* x, int* incx, dcomplex* tau );

void FLA_C2F( zgehrd )( int* n, int* ilo, int* ihi, dcomplex* a, int* lda, dcomplex* tau, dcomplex* work, int* lwork, int* info );
void FLA_C2F( zgehd2 )( int* n, int* ilo, int* ihi, dcomplex* a, int* lda, dcomplex* tau, dcomplex* work, int* info );

void FLA_C2F( ztrtri )( char* uplo, char* diag, int* n, dcomplex* a, int* lda, int* info );
void FLA_C2F( ztrti2 )( char* uplo, char* diag, int* n, dcomplex* a, int* lda, int* info );

void FLA_C2F( zlauum )( char* uplo, int* n, dcomplex* a, int* lda, int* info );
void FLA_C2F( zlauu2 )( char* uplo, int* n, dcomplex* a, int* lda, int* info );

void FLA_C2F( ztrsyl )( char* transa, char* transb, int* isgn, int* m, int* n, dcomplex* a, int* lda, dcomplex* b, int* ldb, dcomplex* c, int* ldc, double* scale, int* info );

void FLA_C2F( zlaswp )( int* n, dcomplex* a, int* lda, int* k1, int* k2, int* ipiv, int* incx );
void FLA_C2F( zlarft )( char* direct, char* storev, int* n, int* k, dcomplex* v, int* ldv, dcomplex* tau, dcomplex* t, int* ldt );
void FLA_C2F( zlarfg )( int* n, dcomplex* alpha, dcomplex* x, int* incx, dcomplex* tau );
void FLA_C2F( zlaset )( char* uplo, int* m, int* n, dcomplex* alpha, dcomplex* beta, dcomplex* a, int* lda );

void FLA_C2F( zunmqr )( char* side, char* trans, int* m, int* n, int* k, dcomplex* a, int* lda, dcomplex* tau, dcomplex* c, int* ldc, dcomplex* work, int* lwork, int* info );
void FLA_C2F( zunm2r )( char* side, char* trans, int* m, int* n, int* k, dcomplex* a, int* lda, dcomplex* tau, dcomplex* c, int* ldc, dcomplex* work, int* info );
void FLA_C2F( zunmlq )( char* side, char* trans, int* m, int* n, int* k, dcomplex* a, int* lda, dcomplex* tau, dcomplex* c, int* ldc, dcomplex* work, int* lwork, int* info );
void FLA_C2F( zunml2 )( char* side, char* trans, int* m, int* n, int* k, dcomplex* a, int* lda, dcomplex* tau, dcomplex* c, int* ldc, dcomplex* work, int* info );
