/*
   libflame
   An object-based infrastructure for developing high-performance
   dense linear algebra libraries.

   Copyright (C) 2009, The University of Texas

   libflame is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as
   published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   libflame is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with libflame; if you did not receive a copy, see
   http://www.gnu.org/licenses/.

   For more information, please contact us at flame@cs.utexas.edu or
   send mail to:

   Field G. Van Zee and/or
   Robert A. van de Geijn
   The University of Texas at Austin
   Department of Computer Sciences
   1 University Station C0500
   Austin TX 78712
*/

#define cbdsqr	CBDSQR
#define cgbbrd	CGBBRD
#define cgbcon	CGBCON
#define cgbequ	CGBEQU
#define cgbrfs	CGBRFS
#define cgbsv	CGBSV
#define cgbsvx	CGBSVX
#define cgbtf2	CGBTF2
#define cgbtrf	CGBTRF
#define cgbtrs	CGBTRS
#define cgebak	CGEBAK
#define cgebal	CGEBAL
#define cgebd2	CGEBD2
#define cgebrd	CGEBRD
#define cgecon	CGECON
#define cgeequ	CGEEQU
#define cgees	CGEES
#define cgeesx	CGEESX
#define cgeev	CGEEV
#define cgeevx	CGEEVX
#define cgegs	CGEGS
#define cgegv	CGEGV
#define cgehd2	CGEHD2
#define cgehrd	CGEHRD
#define cgelq2	CGELQ2
#define cgelqf	CGELQF
#define cgels	CGELS
#define cgelsd	CGELSD
#define cgelss	CGELSS
#define cgelsx	CGELSX
#define cgelsy	CGELSY
#define cgeql2	CGEQL2
#define cgeqlf	CGEQLF
#define cgeqp3	CGEQP3
#define cgeqpf	CGEQPF
#define cgeqr2	CGEQR2
#define cgeqrf	CGEQRF
#define cgerfs	CGERFS
#define cgerq2	CGERQ2
#define cgerqf	CGERQF
#define cgesc2	CGESC2
#define cgesdd	CGESDD
#define cgesv	CGESV
#define cgesvd	CGESVD
#define cgesvx	CGESVX
#define cgetc2	CGETC2
#define cgetf2	CGETF2
#define cgetrf	CGETRF
#define cgetri	CGETRI
#define cgetrs	CGETRS
#define cggbak	CGGBAK
#define cggbal	CGGBAL
#define cgges	CGGES
#define cggesx	CGGESX
#define cggev	CGGEV
#define cggevx	CGGEVX
#define cggglm	CGGGLM
#define cgghrd	CGGHRD
#define cgglse	CGGLSE
#define cggqrf	CGGQRF
#define cggrqf	CGGRQF
#define cggsvd	CGGSVD
#define cggsvp	CGGSVP
#define cgtcon	CGTCON
#define cgtrfs	CGTRFS
#define cgtsv	CGTSV
#define cgtsvx	CGTSVX
#define cgttrf	CGTTRF
#define cgttrs	CGTTRS
#define cgtts2	CGTTS2
#define chbev	CHBEV
#define chbevd	CHBEVD
#define chbevx	CHBEVX
#define chbgst	CHBGST
#define chbgv	CHBGV
#define chbgvd	CHBGVD
#define chbgvx	CHBGVX
#define chbtrd	CHBTRD
#define checon	CHECON
#define cheev	CHEEV
#define cheevd	CHEEVD
#define cheevr	CHEEVR
#define cheevx	CHEEVX
#define chegs2	CHEGS2
#define chegst	CHEGST
#define chegv	CHEGV
#define chegvd	CHEGVD
#define chegvx	CHEGVX
#define cherfs	CHERFS
#define chesv	CHESV
#define chesvx	CHESVX
#define chetd2	CHETD2
#define chetf2	CHETF2
#define chetrd	CHETRD
#define chetrf	CHETRF
#define chetri	CHETRI
#define chetrs	CHETRS
#define chgeqz	CHGEQZ
#define chpcon	CHPCON
#define chpev	CHPEV
#define chpevd	CHPEVD
#define chpevx	CHPEVX
#define chpgst	CHPGST
#define chpgv	CHPGV
#define chpgvd	CHPGVD
#define chpgvx	CHPGVX
#define chprfs	CHPRFS
#define chpsv	CHPSV
#define chpsvx	CHPSVX
#define chptrd	CHPTRD
#define chptrf	CHPTRF
#define chptri	CHPTRI
#define chptrs	CHPTRS
#define chsein	CHSEIN
#define chseqr	CHSEQR
#define clabrd	CLABRD
#define clacgv	CLACGV
#define clacon	CLACON
#define clacp2	CLACP2
#define clacpy	CLACPY
#define clacrm	CLACRM
#define clacrt	CLACRT
#define cladiv	CLADIV
#define claed0	CLAED0
#define claed7	CLAED7
#define claed8	CLAED8
#define claein	CLAEIN
#define claesy	CLAESY
#define claev2	CLAEV2
#define clags2	CLAGS2
#define clagtm	CLAGTM
#define clahef	CLAHEF
#define clahqr	CLAHQR
#define clahrd	CLAHRD
#define claic1	CLAIC1
#define clals0	CLALS0
#define clalsa	CLALSA
#define clalsd	CLALSD
#define clangb	CLANGB
#define clange	CLANGE
#define clangt	CLANGT
#define clanhb	CLANHB
#define clanhe	CLANHE
#define clanhp	CLANHP
#define clanhs	CLANHS
#define clanht	CLANHT
#define clansb	CLANSB
#define clansp	CLANSP
#define clansy	CLANSY
#define clantb	CLANTB
#define clantp	CLANTP
#define clantr	CLANTR
#define clapll	CLAPLL
#define clapmt	CLAPMT
#define claqgb	CLAQGB
#define claqge	CLAQGE
#define claqhb	CLAQHB
#define claqhe	CLAQHE
#define claqhp	CLAQHP
#define claqp2	CLAQP2
#define claqps	CLAQPS
#define claqsb	CLAQSB
#define claqsp	CLAQSP
#define claqsy	CLAQSY
#define clar1v	CLAR1V
#define clar2v	CLAR2V
#define clarcm	CLARCM
#define clarf	CLARF
#define clarfb	CLARFB
#define clarfg	CLARFG
#define clarft	CLARFT
#define clarfx	CLARFX
#define clargv	CLARGV
#define clarnv	CLARNV
#define clarrv	CLARRV
#define clartg	CLARTG
#define clartv	CLARTV
#define clarz	CLARZ
#define clarzb	CLARZB
#define clarzt	CLARZT
#define clascl	CLASCL
#define claset	CLASET
#define clasr	CLASR
#define classq	CLASSQ
#define claswp	CLASWP
#define clasyf	CLASYF
#define clatbs	CLATBS
#define clatdf	CLATDF
#define clatps	CLATPS
#define clatrd	CLATRD
#define clatrs	CLATRS
#define clatrz	CLATRZ
#define clatzm	CLATZM
#define clauu2	CLAUU2
#define clauum	CLAUUM
#define cpbcon	CPBCON
#define cpbequ	CPBEQU
#define cpbrfs	CPBRFS
#define cpbstf	CPBSTF
#define cpbsv	CPBSV
#define cpbsvx	CPBSVX
#define cpbtf2	CPBTF2
#define cpbtrf	CPBTRF
#define cpbtrs	CPBTRS
#define cpocon	CPOCON
#define cpoequ	CPOEQU
#define cporfs	CPORFS
#define cposv	CPOSV
#define cposvx	CPOSVX
#define cpotf2	CPOTF2
#define cpotrf	CPOTRF
#define cpotri	CPOTRI
#define cpotrs	CPOTRS
#define cppcon	CPPCON
#define cppequ	CPPEQU
#define cpprfs	CPPRFS
#define cppsv	CPPSV
#define cppsvx	CPPSVX
#define cpptrf	CPPTRF
#define cpptri	CPPTRI
#define cpptrs	CPPTRS
#define cptcon	CPTCON
#define cpteqr	CPTEQR
#define cptrfs	CPTRFS
#define cptsv	CPTSV
#define cptsvx	CPTSVX
#define cpttrf	CPTTRF
#define cpttrs	CPTTRS
#define cptts2	CPTTS2
#define crot	CROT
#define cspcon	CSPCON
#define cspmv	CSPMV
#define cspr	CSPR
#define csprfs	CSPRFS
#define cspsv	CSPSV
#define cspsvx	CSPSVX
#define csptrf	CSPTRF
#define csptri	CSPTRI
#define csptrs	CSPTRS
#define csrot	CSROT
#define csrscl	CSRSCL
#define cstedc	CSTEDC
#define cstegr	CSTEGR
#define cstein	CSTEIN
#define csteqr	CSTEQR
#define csycon	CSYCON
#define csymv	CSYMV
#define csyr	CSYR
#define csyrfs	CSYRFS
#define csysv	CSYSV
#define csysvx	CSYSVX
#define csytf2	CSYTF2
#define csytrf	CSYTRF
#define csytri	CSYTRI
#define csytrs	CSYTRS
#define ctbcon	CTBCON
#define ctbrfs	CTBRFS
#define ctbtrs	CTBTRS
#define ctgevc	CTGEVC
#define ctgex2	CTGEX2
#define ctgexc	CTGEXC
#define ctgsen	CTGSEN
#define ctgsja	CTGSJA
#define ctgsna	CTGSNA
#define ctgsy2	CTGSY2
#define ctgsyl	CTGSYL
#define ctpcon	CTPCON
#define ctprfs	CTPRFS
#define ctptri	CTPTRI
#define ctptrs	CTPTRS
#define ctrcon	CTRCON
#define ctrevc	CTREVC
#define ctrexc	CTREXC
#define ctrrfs	CTRRFS
#define ctrsen	CTRSEN
#define ctrsna	CTRSNA
#define ctrsyl	CTRSYL
#define ctrti2	CTRTI2
#define ctrtri	CTRTRI
#define ctrtrs	CTRTRS
#define ctzrqf	CTZRQF
#define ctzrzf	CTZRZF
#define cung2l	CUNG2L
#define cung2r	CUNG2R
#define cungbr	CUNGBR
#define cunghr	CUNGHR
#define cungl2	CUNGL2
#define cunglq	CUNGLQ
#define cungql	CUNGQL
#define cungqr	CUNGQR
#define cungr2	CUNGR2
#define cungrq	CUNGRQ
#define cungtr	CUNGTR
#define cunm2l	CUNM2L
#define cunm2r	CUNM2R
#define cunmbr	CUNMBR
#define cunmhr	CUNMHR
#define cunml2	CUNML2
#define cunmlq	CUNMLQ
#define cunmql	CUNMQL
#define cunmqr	CUNMQR
#define cunmr2	CUNMR2
#define cunmr3	CUNMR3
#define cunmrq	CUNMRQ
#define cunmrz	CUNMRZ
#define cunmtr	CUNMTR
#define cupgtr	CUPGTR
#define cupmtr	CUPMTR
#define dbdsdc	DBDSDC
#define dbdsqr	DBDSQR
#define ddisna	DDISNA
#define dgbbrd	DGBBRD
#define dgbcon	DGBCON
#define dgbequ	DGBEQU
#define dgbrfs	DGBRFS
#define dgbsv	DGBSV
#define dgbsvx	DGBSVX
#define dgbtf2	DGBTF2
#define dgbtrf	DGBTRF
#define dgbtrs	DGBTRS
#define dgebak	DGEBAK
#define dgebal	DGEBAL
#define dgebd2	DGEBD2
#define dgebrd	DGEBRD
#define dgecon	DGECON
#define dgeequ	DGEEQU
#define dgees	DGEES
#define dgeesx	DGEESX
#define dgeev	DGEEV
#define dgeevx	DGEEVX
#define dgegs	DGEGS
#define dgegv	DGEGV
#define dgehd2	DGEHD2
#define dgehrd	DGEHRD
#define dgelq2	DGELQ2
#define dgelqf	DGELQF
#define dgels	DGELS
#define dgelsd	DGELSD
#define dgelss	DGELSS
#define dgelsx	DGELSX
#define dgelsy	DGELSY
#define dgeql2	DGEQL2
#define dgeqlf	DGEQLF
#define dgeqp3	DGEQP3
#define dgeqpf	DGEQPF
#define dgeqr2	DGEQR2
#define dgeqrf	DGEQRF
#define dgerfs	DGERFS
#define dgerq2	DGERQ2
#define dgerqf	DGERQF
#define dgesc2	DGESC2
#define dgesdd	DGESDD
#define dgesv	DGESV
#define dgesvd	DGESVD
#define dgesvx	DGESVX
#define dgetc2	DGETC2
#define dgetf2	DGETF2
#define dgetrf	DGETRF
#define dgetri	DGETRI
#define dgetrs	DGETRS
#define dggbak	DGGBAK
#define dggbal	DGGBAL
#define dgges	DGGES
#define dggesx	DGGESX
#define dggev	DGGEV
#define dggevx	DGGEVX
#define dggglm	DGGGLM
#define dgghrd	DGGHRD
#define dgglse	DGGLSE
#define dggqrf	DGGQRF
#define dggrqf	DGGRQF
#define dggsvd	DGGSVD
#define dggsvp	DGGSVP
#define dgtcon	DGTCON
#define dgtrfs	DGTRFS
#define dgtsv	DGTSV
#define dgtsvx	DGTSVX
#define dgttrf	DGTTRF
#define dgttrs	DGTTRS
#define dgtts2	DGTTS2
#define dhgeqz	DHGEQZ
#define dhsein	DHSEIN
#define dhseqr	DHSEQR
#define dlabad	DLABAD
#define dlabrd	DLABRD
#define dlacon	DLACON
#define dlacpy	DLACPY
#define dladiv	DLADIV
#define dlae2	DLAE2
#define dlaebz	DLAEBZ
#define dlaed0	DLAED0
#define dlaed1	DLAED1
#define dlaed2	DLAED2
#define dlaed3	DLAED3
#define dlaed4	DLAED4
#define dlaed5	DLAED5
#define dlaed6	DLAED6
#define dlaed7	DLAED7
#define dlaed8	DLAED8
#define dlaed9	DLAED9
#define dlaeda	DLAEDA
#define dlaein	DLAEIN
#define dlaev2	DLAEV2
#define dlaexc	DLAEXC
#define dlag2	DLAG2
#define dlags2	DLAGS2
#define dlagtf	DLAGTF
#define dlagtm	DLAGTM
#define dlagts	DLAGTS
#define dlagv2	DLAGV2
#define dlahqr	DLAHQR
#define dlahrd	DLAHRD
#define dlaic1	DLAIC1
#define dlaln2	DLALN2
#define dlals0	DLALS0
#define dlalsa	DLALSA
#define dlalsd	DLALSD
#define dlamch	DLAMCH
#define dlamrg	DLAMRG
#define dlangb	DLANGB
#define dlange	DLANGE
#define dlangt	DLANGT
#define dlanhs	DLANHS
#define dlansb	DLANSB
#define dlansp	DLANSP
#define dlanst	DLANST
#define dlansy	DLANSY
#define dlantb	DLANTB
#define dlantp	DLANTP
#define dlantr	DLANTR
#define dlanv2	DLANV2
#define dlapll	DLAPLL
#define dlapmt	DLAPMT
#define dlapy2	DLAPY2
#define dlapy3	DLAPY3
#define dlaqgb	DLAQGB
#define dlaqge	DLAQGE
#define dlaqp2	DLAQP2
#define dlaqps	DLAQPS
#define dlaqsb	DLAQSB
#define dlaqsp	DLAQSP
#define dlaqsy	DLAQSY
#define dlaqtr	DLAQTR
#define dlar1v	DLAR1V
#define dlar2v	DLAR2V
#define dlarf	DLARF
#define dlarfb	DLARFB
#define dlarfg	DLARFG
#define dlarft	DLARFT
#define dlarfx	DLARFX
#define dlargv	DLARGV
#define dlarnv	DLARNV
#define dlarrb	DLARRB
#define dlarre	DLARRE
#define dlarrf	DLARRF
#define dlarrv	DLARRV
#define dlartg	DLARTG
#define dlartv	DLARTV
#define dlaruv	DLARUV
#define dlarz	DLARZ
#define dlarzb	DLARZB
#define dlarzt	DLARZT
#define dlas2	DLAS2
#define dlascl	DLASCL
#define dlasd0	DLASD0
#define dlasd1	DLASD1
#define dlasd2	DLASD2
#define dlasd3	DLASD3
#define dlasd4	DLASD4
#define dlasd5	DLASD5
#define dlasd6	DLASD6
#define dlasd7	DLASD7
#define dlasd8	DLASD8
#define dlasd9	DLASD9
#define dlasda	DLASDA
#define dlasdq	DLASDQ
#define dlasdt	DLASDT
#define dlaset	DLASET
#define dlasq1	DLASQ1
#define dlasq2	DLASQ2
#define dlasq3	DLASQ3
#define dlasq4	DLASQ4
#define dlasq5	DLASQ5
#define dlasq6	DLASQ6
#define dlasr	DLASR
#define dlasrt	DLASRT
#define dlassq	DLASSQ
#define dlasv2	DLASV2
#define dlaswp	DLASWP
#define dlasy2	DLASY2
#define dlasyf	DLASYF
#define dlatbs	DLATBS
#define dlatdf	DLATDF
#define dlatps	DLATPS
#define dlatrd	DLATRD
#define dlatrs	DLATRS
#define dlatrz	DLATRZ
#define dlatzm	DLATZM
#define dlauu2	DLAUU2
#define dlauum	DLAUUM
#define dopgtr	DOPGTR
#define dopmtr	DOPMTR
#define dorg2l	DORG2L
#define dorg2r	DORG2R
#define dorgbr	DORGBR
#define dorghr	DORGHR
#define dorgl2	DORGL2
#define dorglq	DORGLQ
#define dorgql	DORGQL
#define dorgqr	DORGQR
#define dorgr2	DORGR2
#define dorgrq	DORGRQ
#define dorgtr	DORGTR
#define dorm2l	DORM2L
#define dorm2r	DORM2R
#define dormbr	DORMBR
#define dormhr	DORMHR
#define dorml2	DORML2
#define dormlq	DORMLQ
#define dormql	DORMQL
#define dormqr	DORMQR
#define dormr2	DORMR2
#define dormr3	DORMR3
#define dormrq	DORMRQ
#define dormrz	DORMRZ
#define dormtr	DORMTR
#define dpbcon	DPBCON
#define dpbequ	DPBEQU
#define dpbrfs	DPBRFS
#define dpbstf	DPBSTF
#define dpbsv	DPBSV
#define dpbsvx	DPBSVX
#define dpbtf2	DPBTF2
#define dpbtrf	DPBTRF
#define dpbtrs	DPBTRS
#define dpocon	DPOCON
#define dpoequ	DPOEQU
#define dporfs	DPORFS
#define dposv	DPOSV
#define dposvx	DPOSVX
#define dpotf2	DPOTF2
#define dpotrf	DPOTRF
#define dpotri	DPOTRI
#define dpotrs	DPOTRS
#define dppcon	DPPCON
#define dppequ	DPPEQU
#define dpprfs	DPPRFS
#define dppsv	DPPSV
#define dppsvx	DPPSVX
#define dpptrf	DPPTRF
#define dpptri	DPPTRI
#define dpptrs	DPPTRS
#define dptcon	DPTCON
#define dpteqr	DPTEQR
#define dptrfs	DPTRFS
#define dptsv	DPTSV
#define dptsvx	DPTSVX
#define dpttrf	DPTTRF
#define dpttrs	DPTTRS
#define dptts2	DPTTS2
#define drscl	DRSCL
#define dsbev	DSBEV
#define dsbevd	DSBEVD
#define dsbevx	DSBEVX
#define dsbgst	DSBGST
#define dsbgv	DSBGV
#define dsbgvd	DSBGVD
#define dsbgvx	DSBGVX
#define dsbtrd	DSBTRD
#define dsecnd	DSECND
#define dspcon	DSPCON
#define dspev	DSPEV
#define dspevd	DSPEVD
#define dspevx	DSPEVX
#define dspgst	DSPGST
#define dspgv	DSPGV
#define dspgvd	DSPGVD
#define dspgvx	DSPGVX
#define dsprfs	DSPRFS
#define dspsv	DSPSV
#define dspsvx	DSPSVX
#define dsptrd	DSPTRD
#define dsptrf	DSPTRF
#define dsptri	DSPTRI
#define dsptrs	DSPTRS
#define dstebz	DSTEBZ
#define dstedc	DSTEDC
#define dstegr	DSTEGR
#define dstein	DSTEIN
#define dsteqr	DSTEQR
#define dsterf	DSTERF
#define dstev	DSTEV
#define dstevd	DSTEVD
#define dstevr	DSTEVR
#define dstevx	DSTEVX
#define dsycon	DSYCON
#define dsyev	DSYEV
#define dsyevd	DSYEVD
#define dsyevr	DSYEVR
#define dsyevx	DSYEVX
#define dsygs2	DSYGS2
#define dsygst	DSYGST
#define dsygv	DSYGV
#define dsygvd	DSYGVD
#define dsygvx	DSYGVX
#define dsyrfs	DSYRFS
#define dsysv	DSYSV
#define dsysvx	DSYSVX
#define dsytd2	DSYTD2
#define dsytf2	DSYTF2
#define dsytrd	DSYTRD
#define dsytrf	DSYTRF
#define dsytri	DSYTRI
#define dsytrs	DSYTRS
#define dtbcon	DTBCON
#define dtbrfs	DTBRFS
#define dtbtrs	DTBTRS
#define dtgevc	DTGEVC
#define dtgex2	DTGEX2
#define dtgexc	DTGEXC
#define dtgsen	DTGSEN
#define dtgsja	DTGSJA
#define dtgsna	DTGSNA
#define dtgsy2	DTGSY2
#define dtgsyl	DTGSYL
#define dtpcon	DTPCON
#define dtprfs	DTPRFS
#define dtptri	DTPTRI
#define dtptrs	DTPTRS
#define dtrcon	DTRCON
#define dtrevc	DTREVC
#define dtrexc	DTREXC
#define dtrrfs	DTRRFS
#define dtrsen	DTRSEN
#define dtrsna	DTRSNA
#define dtrsyl	DTRSYL
#define dtrti2	DTRTI2
#define dtrtri	DTRTRI
#define dtrtrs	DTRTRS
#define dtzrqf	DTZRQF
#define dtzrzf	DTZRZF
#define dzsum1	DZSUM1
#define icmax1	ICMAX1
#define ieeeck	IEEECK
#define ilaenv	ILAENV
#define izmax1	IZMAX1
#define lsame	LSAME
#define lsamen	LSAMEN
#define sbdsdc	SBDSDC
#define sbdsqr	SBDSQR
#define scsum1	SCSUM1
#define sdisna	SDISNA
#define second	SECOND
#define sgbbrd	SGBBRD
#define sgbcon	SGBCON
#define sgbequ	SGBEQU
#define sgbrfs	SGBRFS
#define sgbsv	SGBSV
#define sgbsvx	SGBSVX
#define sgbtf2	SGBTF2
#define sgbtrf	SGBTRF
#define sgbtrs	SGBTRS
#define sgebak	SGEBAK
#define sgebal	SGEBAL
#define sgebd2	SGEBD2
#define sgebrd	SGEBRD
#define sgecon	SGECON
#define sgeequ	SGEEQU
#define sgees	SGEES
#define sgeesx	SGEESX
#define sgeev	SGEEV
#define sgeevx	SGEEVX
#define sgegs	SGEGS
#define sgegv	SGEGV
#define sgehd2	SGEHD2
#define sgehrd	SGEHRD
#define sgelq2	SGELQ2
#define sgelqf	SGELQF
#define sgels	SGELS
#define sgelsd	SGELSD
#define sgelss	SGELSS
#define sgelsx	SGELSX
#define sgelsy	SGELSY
#define sgeql2	SGEQL2
#define sgeqlf	SGEQLF
#define sgeqp3	SGEQP3
#define sgeqpf	SGEQPF
#define sgeqr2	SGEQR2
#define sgeqrf	SGEQRF
#define sgerfs	SGERFS
#define sgerq2	SGERQ2
#define sgerqf	SGERQF
#define sgesc2	SGESC2
#define sgesdd	SGESDD
#define sgesv	SGESV
#define sgesvd	SGESVD
#define sgesvx	SGESVX
#define sgetc2	SGETC2
#define sgetf2	SGETF2
#define sgetrf	SGETRF
#define sgetri	SGETRI
#define sgetrs	SGETRS
#define sggbak	SGGBAK
#define sggbal	SGGBAL
#define sgges	SGGES
#define sggesx	SGGESX
#define sggev	SGGEV
#define sggevx	SGGEVX
#define sggglm	SGGGLM
#define sgghrd	SGGHRD
#define sgglse	SGGLSE
#define sggqrf	SGGQRF
#define sggrqf	SGGRQF
#define sggsvd	SGGSVD
#define sggsvp	SGGSVP
#define sgtcon	SGTCON
#define sgtrfs	SGTRFS
#define sgtsv	SGTSV
#define sgtsvx	SGTSVX
#define sgttrf	SGTTRF
#define sgttrs	SGTTRS
#define sgtts2	SGTTS2
#define shgeqz	SHGEQZ
#define shsein	SHSEIN
#define shseqr	SHSEQR
#define slabad	SLABAD
#define slabrd	SLABRD
#define slacon	SLACON
#define slacpy	SLACPY
#define sladiv	SLADIV
#define slae2	SLAE2
#define slaebz	SLAEBZ
#define slaed0	SLAED0
#define slaed1	SLAED1
#define slaed2	SLAED2
#define slaed3	SLAED3
#define slaed4	SLAED4
#define slaed5	SLAED5
#define slaed6	SLAED6
#define slaed7	SLAED7
#define slaed8	SLAED8
#define slaed9	SLAED9
#define slaeda	SLAEDA
#define slaein	SLAEIN
#define slaev2	SLAEV2
#define slaexc	SLAEXC
#define slag2	SLAG2
#define slags2	SLAGS2
#define slagtf	SLAGTF
#define slagtm	SLAGTM
#define slagts	SLAGTS
#define slagv2	SLAGV2
#define slahqr	SLAHQR
#define slahrd	SLAHRD
#define slaic1	SLAIC1
#define slaln2	SLALN2
#define slals0	SLALS0
#define slalsa	SLALSA
#define slalsd	SLALSD
#define slamch	SLAMCH
#define slamrg	SLAMRG
#define slangb	SLANGB
#define slange	SLANGE
#define slangt	SLANGT
#define slanhs	SLANHS
#define slansb	SLANSB
#define slansp	SLANSP
#define slanst	SLANST
#define slansy	SLANSY
#define slantb	SLANTB
#define slantp	SLANTP
#define slantr	SLANTR
#define slanv2	SLANV2
#define slapll	SLAPLL
#define slapmt	SLAPMT
#define slapy2	SLAPY2
#define slapy3	SLAPY3
#define slaqgb	SLAQGB
#define slaqge	SLAQGE
#define slaqp2	SLAQP2
#define slaqps	SLAQPS
#define slaqsb	SLAQSB
#define slaqsp	SLAQSP
#define slaqsy	SLAQSY
#define slaqtr	SLAQTR
#define slar1v	SLAR1V
#define slar2v	SLAR2V
#define slarf	SLARF
#define slarfb	SLARFB
#define slarfg	SLARFG
#define slarft	SLARFT
#define slarfx	SLARFX
#define slargv	SLARGV
#define slarnv	SLARNV
#define slarrb	SLARRB
#define slarre	SLARRE
#define slarrf	SLARRF
#define slarrv	SLARRV
#define slartg	SLARTG
#define slartv	SLARTV
#define slaruv	SLARUV
#define slarz	SLARZ
#define slarzb	SLARZB
#define slarzt	SLARZT
#define slas2	SLAS2
#define slascl	SLASCL
#define slasd0	SLASD0
#define slasd1	SLASD1
#define slasd2	SLASD2
#define slasd3	SLASD3
#define slasd4	SLASD4
#define slasd5	SLASD5
#define slasd6	SLASD6
#define slasd7	SLASD7
#define slasd8	SLASD8
#define slasd9	SLASD9
#define slasda	SLASDA
#define slasdq	SLASDQ
#define slasdt	SLASDT
#define slaset	SLASET
#define slasq1	SLASQ1
#define slasq2	SLASQ2
#define slasq3	SLASQ3
#define slasq4	SLASQ4
#define slasq5	SLASQ5
#define slasq6	SLASQ6
#define slasr	SLASR
#define slasrt	SLASRT
#define slassq	SLASSQ
#define slasv2	SLASV2
#define slaswp	SLASWP
#define slasy2	SLASY2
#define slasyf	SLASYF
#define slatbs	SLATBS
#define slatdf	SLATDF
#define slatps	SLATPS
#define slatrd	SLATRD
#define slatrs	SLATRS
#define slatrz	SLATRZ
#define slatzm	SLATZM
#define slauu2	SLAUU2
#define slauum	SLAUUM
#define sopgtr	SOPGTR
#define sopmtr	SOPMTR
#define sorg2l	SORG2L
#define sorg2r	SORG2R
#define sorgbr	SORGBR
#define sorghr	SORGHR
#define sorgl2	SORGL2
#define sorglq	SORGLQ
#define sorgql	SORGQL
#define sorgqr	SORGQR
#define sorgr2	SORGR2
#define sorgrq	SORGRQ
#define sorgtr	SORGTR
#define sorm2l	SORM2L
#define sorm2r	SORM2R
#define sormbr	SORMBR
#define sormhr	SORMHR
#define sorml2	SORML2
#define sormlq	SORMLQ
#define sormql	SORMQL
#define sormqr	SORMQR
#define sormr2	SORMR2
#define sormr3	SORMR3
#define sormrq	SORMRQ
#define sormrz	SORMRZ
#define sormtr	SORMTR
#define spbcon	SPBCON
#define spbequ	SPBEQU
#define spbrfs	SPBRFS
#define spbstf	SPBSTF
#define spbsv	SPBSV
#define spbsvx	SPBSVX
#define spbtf2	SPBTF2
#define spbtrf	SPBTRF
#define spbtrs	SPBTRS
#define spocon	SPOCON
#define spoequ	SPOEQU
#define sporfs	SPORFS
#define sposv	SPOSV
#define sposvx	SPOSVX
#define spotf2	SPOTF2
#define spotrf	SPOTRF
#define spotri	SPOTRI
#define spotrs	SPOTRS
#define sppcon	SPPCON
#define sppequ	SPPEQU
#define spprfs	SPPRFS
#define sppsv	SPPSV
#define sppsvx	SPPSVX
#define spptrf	SPPTRF
#define spptri	SPPTRI
#define spptrs	SPPTRS
#define sptcon	SPTCON
#define spteqr	SPTEQR
#define sptrfs	SPTRFS
#define sptsv	SPTSV
#define sptsvx	SPTSVX
#define spttrf	SPTTRF
#define spttrs	SPTTRS
#define sptts2	SPTTS2
#define srscl	SRSCL
#define ssbev	SSBEV
#define ssbevd	SSBEVD
#define ssbevx	SSBEVX
#define ssbgst	SSBGST
#define ssbgv	SSBGV
#define ssbgvd	SSBGVD
#define ssbgvx	SSBGVX
#define ssbtrd	SSBTRD
#define sspcon	SSPCON
#define sspev	SSPEV
#define sspevd	SSPEVD
#define sspevx	SSPEVX
#define sspgst	SSPGST
#define sspgv	SSPGV
#define sspgvd	SSPGVD
#define sspgvx	SSPGVX
#define ssprfs	SSPRFS
#define sspsv	SSPSV
#define sspsvx	SSPSVX
#define ssptrd	SSPTRD
#define ssptrf	SSPTRF
#define ssptri	SSPTRI
#define ssptrs	SSPTRS
#define sstebz	SSTEBZ
#define sstedc	SSTEDC
#define sstegr	SSTEGR
#define sstein	SSTEIN
#define ssteqr	SSTEQR
#define ssterf	SSTERF
#define sstev	SSTEV
#define sstevd	SSTEVD
#define sstevr	SSTEVR
#define sstevx	SSTEVX
#define ssycon	SSYCON
#define ssyev	SSYEV
#define ssyevd	SSYEVD
#define ssyevr	SSYEVR
#define ssyevx	SSYEVX
#define ssygs2	SSYGS2
#define ssygst	SSYGST
#define ssygv	SSYGV
#define ssygvd	SSYGVD
#define ssygvx	SSYGVX
#define ssyrfs	SSYRFS
#define ssysv	SSYSV
#define ssysvx	SSYSVX
#define ssytd2	SSYTD2
#define ssytf2	SSYTF2
#define ssytrd	SSYTRD
#define ssytrf	SSYTRF
#define ssytri	SSYTRI
#define ssytrs	SSYTRS
#define stbcon	STBCON
#define stbrfs	STBRFS
#define stbtrs	STBTRS
#define stgevc	STGEVC
#define stgex2	STGEX2
#define stgexc	STGEXC
#define stgsen	STGSEN
#define stgsja	STGSJA
#define stgsna	STGSNA
#define stgsy2	STGSY2
#define stgsyl	STGSYL
#define stpcon	STPCON
#define stprfs	STPRFS
#define stptri	STPTRI
#define stptrs	STPTRS
#define strcon	STRCON
#define strevc	STREVC
#define strexc	STREXC
#define strrfs	STRRFS
#define strsen	STRSEN
#define strsna	STRSNA
#define strsyl	STRSYL
#define strti2	STRTI2
#define strtri	STRTRI
#define strtrs	STRTRS
#define stzrqf	STZRQF
#define stzrzf	STZRZF
#define xerbla	XERBLA
#define zbdsqr	ZBDSQR
#define zdrot	ZDROT
#define zdrscl	ZDRSCL
#define zgbbrd	ZGBBRD
#define zgbcon	ZGBCON
#define zgbequ	ZGBEQU
#define zgbrfs	ZGBRFS
#define zgbsv	ZGBSV
#define zgbsvx	ZGBSVX
#define zgbtf2	ZGBTF2
#define zgbtrf	ZGBTRF
#define zgbtrs	ZGBTRS
#define zgebak	ZGEBAK
#define zgebal	ZGEBAL
#define zgebd2	ZGEBD2
#define zgebrd	ZGEBRD
#define zgecon	ZGECON
#define zgeequ	ZGEEQU
#define zgees	ZGEES
#define zgeesx	ZGEESX
#define zgeev	ZGEEV
#define zgeevx	ZGEEVX
#define zgegs	ZGEGS
#define zgegv	ZGEGV
#define zgehd2	ZGEHD2
#define zgehrd	ZGEHRD
#define zgelq2	ZGELQ2
#define zgelqf	ZGELQF
#define zgels	ZGELS
#define zgelsd	ZGELSD
#define zgelss	ZGELSS
#define zgelsx	ZGELSX
#define zgelsy	ZGELSY
#define zgeql2	ZGEQL2
#define zgeqlf	ZGEQLF
#define zgeqp3	ZGEQP3
#define zgeqpf	ZGEQPF
#define zgeqr2	ZGEQR2
#define zgeqrf	ZGEQRF
#define zgerfs	ZGERFS
#define zgerq2	ZGERQ2
#define zgerqf	ZGERQF
#define zgesc2	ZGESC2
#define zgesdd	ZGESDD
#define zgesv	ZGESV
#define zgesvd	ZGESVD
#define zgesvx	ZGESVX
#define zgetc2	ZGETC2
#define zgetf2	ZGETF2
#define zgetrf	ZGETRF
#define zgetri	ZGETRI
#define zgetrs	ZGETRS
#define zggbak	ZGGBAK
#define zggbal	ZGGBAL
#define zgges	ZGGES
#define zggesx	ZGGESX
#define zggev	ZGGEV
#define zggevx	ZGGEVX
#define zggglm	ZGGGLM
#define zgghrd	ZGGHRD
#define zgglse	ZGGLSE
#define zggqrf	ZGGQRF
#define zggrqf	ZGGRQF
#define zggsvd	ZGGSVD
#define zggsvp	ZGGSVP
#define zgtcon	ZGTCON
#define zgtrfs	ZGTRFS
#define zgtsv	ZGTSV
#define zgtsvx	ZGTSVX
#define zgttrf	ZGTTRF
#define zgttrs	ZGTTRS
#define zgtts2	ZGTTS2
#define zhbev	ZHBEV
#define zhbevd	ZHBEVD
#define zhbevx	ZHBEVX
#define zhbgst	ZHBGST
#define zhbgv	ZHBGV
#define zhbgvd	ZHBGVD
#define zhbgvx	ZHBGVX
#define zhbtrd	ZHBTRD
#define zhecon	ZHECON
#define zheev	ZHEEV
#define zheevd	ZHEEVD
#define zheevr	ZHEEVR
#define zheevx	ZHEEVX
#define zhegs2	ZHEGS2
#define zhegst	ZHEGST
#define zhegv	ZHEGV
#define zhegvd	ZHEGVD
#define zhegvx	ZHEGVX
#define zherfs	ZHERFS
#define zhesv	ZHESV
#define zhesvx	ZHESVX
#define zhetd2	ZHETD2
#define zhetf2	ZHETF2
#define zhetrd	ZHETRD
#define zhetrf	ZHETRF
#define zhetri	ZHETRI
#define zhetrs	ZHETRS
#define zhgeqz	ZHGEQZ
#define zhpcon	ZHPCON
#define zhpev	ZHPEV
#define zhpevd	ZHPEVD
#define zhpevx	ZHPEVX
#define zhpgst	ZHPGST
#define zhpgv	ZHPGV
#define zhpgvd	ZHPGVD
#define zhpgvx	ZHPGVX
#define zhprfs	ZHPRFS
#define zhpsv	ZHPSV
#define zhpsvx	ZHPSVX
#define zhptrd	ZHPTRD
#define zhptrf	ZHPTRF
#define zhptri	ZHPTRI
#define zhptrs	ZHPTRS
#define zhsein	ZHSEIN
#define zhseqr	ZHSEQR
#define zlabrd	ZLABRD
#define zlacgv	ZLACGV
#define zlacon	ZLACON
#define zlacp2	ZLACP2
#define zlacpy	ZLACPY
#define zlacrm	ZLACRM
#define zlacrt	ZLACRT
#define zladiv	ZLADIV
#define zlaed0	ZLAED0
#define zlaed7	ZLAED7
#define zlaed8	ZLAED8
#define zlaein	ZLAEIN
#define zlaesy	ZLAESY
#define zlaev2	ZLAEV2
#define zlags2	ZLAGS2
#define zlagtm	ZLAGTM
#define zlahef	ZLAHEF
#define zlahqr	ZLAHQR
#define zlahrd	ZLAHRD
#define zlaic1	ZLAIC1
#define zlals0	ZLALS0
#define zlalsa	ZLALSA
#define zlalsd	ZLALSD
#define zlangb	ZLANGB
#define zlange	ZLANGE
#define zlangt	ZLANGT
#define zlanhb	ZLANHB
#define zlanhe	ZLANHE
#define zlanhp	ZLANHP
#define zlanhs	ZLANHS
#define zlanht	ZLANHT
#define zlansb	ZLANSB
#define zlansp	ZLANSP
#define zlansy	ZLANSY
#define zlantb	ZLANTB
#define zlantp	ZLANTP
#define zlantr	ZLANTR
#define zlapll	ZLAPLL
#define zlapmt	ZLAPMT
#define zlaqgb	ZLAQGB
#define zlaqge	ZLAQGE
#define zlaqhb	ZLAQHB
#define zlaqhe	ZLAQHE
#define zlaqhp	ZLAQHP
#define zlaqp2	ZLAQP2
#define zlaqps	ZLAQPS
#define zlaqsb	ZLAQSB
#define zlaqsp	ZLAQSP
#define zlaqsy	ZLAQSY
#define zlar1v	ZLAR1V
#define zlar2v	ZLAR2V
#define zlarcm	ZLARCM
#define zlarf	ZLARF
#define zlarfb	ZLARFB
#define zlarfg	ZLARFG
#define zlarft	ZLARFT
#define zlarfx	ZLARFX
#define zlargv	ZLARGV
#define zlarnv	ZLARNV
#define zlarrv	ZLARRV
#define zlartg	ZLARTG
#define zlartv	ZLARTV
#define zlarz	ZLARZ
#define zlarzb	ZLARZB
#define zlarzt	ZLARZT
#define zlascl	ZLASCL
#define zlaset	ZLASET
#define zlasr	ZLASR
#define zlassq	ZLASSQ
#define zlaswp	ZLASWP
#define zlasyf	ZLASYF
#define zlatbs	ZLATBS
#define zlatdf	ZLATDF
#define zlatps	ZLATPS
#define zlatrd	ZLATRD
#define zlatrs	ZLATRS
#define zlatrz	ZLATRZ
#define zlatzm	ZLATZM
#define zlauu2	ZLAUU2
#define zlauum	ZLAUUM
#define zpbcon	ZPBCON
#define zpbequ	ZPBEQU
#define zpbrfs	ZPBRFS
#define zpbstf	ZPBSTF
#define zpbsv	ZPBSV
#define zpbsvx	ZPBSVX
#define zpbtf2	ZPBTF2
#define zpbtrf	ZPBTRF
#define zpbtrs	ZPBTRS
#define zpocon	ZPOCON
#define zpoequ	ZPOEQU
#define zporfs	ZPORFS
#define zposv	ZPOSV
#define zposvx	ZPOSVX
#define zpotf2	ZPOTF2
#define zpotrf	ZPOTRF
#define zpotri	ZPOTRI
#define zpotrs	ZPOTRS
#define zppcon	ZPPCON
#define zppequ	ZPPEQU
#define zpprfs	ZPPRFS
#define zppsv	ZPPSV
#define zppsvx	ZPPSVX
#define zpptrf	ZPPTRF
#define zpptri	ZPPTRI
#define zpptrs	ZPPTRS
#define zptcon	ZPTCON
#define zpteqr	ZPTEQR
#define zptrfs	ZPTRFS
#define zptsv	ZPTSV
#define zptsvx	ZPTSVX
#define zpttrf	ZPTTRF
#define zpttrs	ZPTTRS
#define zptts2	ZPTTS2
#define zrot	ZROT
#define zspcon	ZSPCON
#define zspmv	ZSPMV
#define zspr	ZSPR
#define zsprfs	ZSPRFS
#define zspsv	ZSPSV
#define zspsvx	ZSPSVX
#define zsptrf	ZSPTRF
#define zsptri	ZSPTRI
#define zsptrs	ZSPTRS
#define zstedc	ZSTEDC
#define zstegr	ZSTEGR
#define zstein	ZSTEIN
#define zsteqr	ZSTEQR
#define zsycon	ZSYCON
#define zsymv	ZSYMV
#define zsyr	ZSYR
#define zsyrfs	ZSYRFS
#define zsysv	ZSYSV
#define zsysvx	ZSYSVX
#define zsytf2	ZSYTF2
#define zsytrf	ZSYTRF
#define zsytri	ZSYTRI
#define zsytrs	ZSYTRS
#define ztbcon	ZTBCON
#define ztbrfs	ZTBRFS
#define ztbtrs	ZTBTRS
#define ztgevc	ZTGEVC
#define ztgex2	ZTGEX2
#define ztgexc	ZTGEXC
#define ztgsen	ZTGSEN
#define ztgsja	ZTGSJA
#define ztgsna	ZTGSNA
#define ztgsy2	ZTGSY2
#define ztgsyl	ZTGSYL
#define ztpcon	ZTPCON
#define ztprfs	ZTPRFS
#define ztptri	ZTPTRI
#define ztptrs	ZTPTRS
#define ztrcon	ZTRCON
#define ztrevc	ZTREVC
#define ztrexc	ZTREXC
#define ztrrfs	ZTRRFS
#define ztrsen	ZTRSEN
#define ztrsna	ZTRSNA
#define ztrsyl	ZTRSYL
#define ztrti2	ZTRTI2
#define ztrtri	ZTRTRI
#define ztrtrs	ZTRTRS
#define ztzrqf	ZTZRQF
#define ztzrzf	ZTZRZF
#define zung2l	ZUNG2L
#define zung2r	ZUNG2R
#define zungbr	ZUNGBR
#define zunghr	ZUNGHR
#define zungl2	ZUNGL2
#define zunglq	ZUNGLQ
#define zungql	ZUNGQL
#define zungqr	ZUNGQR
#define zungr2	ZUNGR2
#define zungrq	ZUNGRQ
#define zungtr	ZUNGTR
#define zunm2l	ZUNM2L
#define zunm2r	ZUNM2R
#define zunmbr	ZUNMBR
#define zunmhr	ZUNMHR
#define zunml2	ZUNML2
#define zunmlq	ZUNMLQ
#define zunmql	ZUNMQL
#define zunmqr	ZUNMQR
#define zunmr2	ZUNMR2
#define zunmr3	ZUNMR3
#define zunmrq	ZUNMRQ
#define zunmrz	ZUNMRZ
#define zunmtr	ZUNMTR
#define zupgtr	ZUPGTR
#define zupmtr	ZUPMTR
