/*
   libflame
   An object-based infrastructure for developing high-performance
   dense linear algebra libraries.

   Copyright (C) 2009, The University of Texas

   libflame is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as
   published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   libflame is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with libflame; if you did not receive a copy, see
   http://www.gnu.org/licenses/.

   For more information, please contact us at flame@cs.utexas.edu or
   send mail to:

   Field G. Van Zee and/or
   Robert A. van de Geijn
   The University of Texas at Austin
   Department of Computer Sciences
   1 University Station C0500
   Austin TX 78712
*/

float     FLA_random_float( void );
double    FLA_random_double( void );
scomplex  FLA_random_scomplex( void );
dcomplex  FLA_random_dcomplex( void );

FLA_Error FLA_Absolute_square( FLA_Obj alpha );
double    FLA_Clock( void );
FLA_Error FLA_Conjugate( FLA_Obj A );
FLA_Error FLA_Conjugate_r( FLA_Uplo uplo, FLA_Obj A );
FLA_Error FLA_Hermitianize( FLA_Uplo uplo, FLA_Obj A );
FLA_Error FLA_Invert( FLA_Obj alpha );
FLA_Error FLA_Max_abs_value( FLA_Obj A, FLA_Obj amax );
double    FLA_Max_elemwise_diff( FLA_Obj A, FLA_Obj B );
FLA_Error FLA_FLA_Mult_add( FLA_Obj alpha, FLA_Obj beta, FLA_Obj gamma );
FLA_Error FLA_Negate( FLA_Obj x );
FLA_Error FLA_Norm1( FLA_Obj A, FLA_Obj norm );
FLA_Error FLA_Norm_inf( FLA_Obj A, FLA_Obj norm );
FLA_Error FLA_Random_matrix( FLA_Obj A );
FLA_Error FLA_Random_herm_matrix( FLA_Uplo uplo, FLA_Obj A );
FLA_Error FLA_Random_spd_matrix( FLA_Uplo uplo, FLA_Obj A );
FLA_Error FLA_Random_tri_matrix( FLA_Uplo uplo, FLA_Diag diag, FLA_Obj A );
FLA_Error FLA_Shift_pivots_to_check( FLA_Pivot_type ptype, FLA_Obj ipiv );
FLA_Error FLA_Sqrt( FLA_Obj alpha );
FLA_Error FLA_Symmetrize( FLA_Uplo uplo, FLA_Obj A );
FLA_Error FLA_Triangularize( FLA_Uplo uplo, FLA_Diag diag, FLA_Obj A );
FLA_Error FLA_Transpose( FLA_Obj A );

void      FLA_F2C( fla_absolute_square_f    ) ( F_INT *alpha, F_INT *IERROR );
double    FLA_F2C( fla_clock_f              ) ( void );
void      FLA_F2C( fla_conjugate_f          ) ( F_INT *A, F_INT *IERROR );
void      FLA_F2C( fla_conjugate_r_f        ) ( F_INT *uplo, F_INT *A, F_INT *IERROR );
void      FLA_F2C( fla_hermitianize_f       ) ( F_INT *uplo, F_INT *A, F_INT *IERROR );
void      FLA_F2C( fla_invert_f             ) ( F_INT *alpha, F_INT *IERROR );
void      FLA_F2C( fla_max_abs_value_f      ) ( F_INT *A, F_INT *amax, F_INT *IERROR );
double    FLA_F2C( fla_max_elemwise_diff_f  ) ( F_INT *A, F_INT *B );
void      FLA_F2C( fla_mult_add_f           ) ( F_INT *alpha, F_INT *beta, F_INT *gamma, F_INT *IERROR );
void      FLA_F2C( fla_negate_f             ) ( F_INT *x, F_INT *IERROR );
void      FLA_F2C( fla_norm1_f              ) ( F_INT *A, F_INT *norm, F_INT *IERROR );
void      FLA_F2C( fla_norm_inf_f           ) ( F_INT *A, F_INT *norm, F_INT *IERROR );
void      FLA_F2C( fla_random_matrix_f      ) ( F_INT *A, F_INT *IERROR );
void      FLA_F2C( fla_random_herm_matrix_f ) ( F_INT *uplo, F_INT *A, F_INT *IERROR );
void      FLA_F2C( fla_random_spd_matrix_f  ) ( F_INT *uplo, F_INT *A, F_INT *IERROR );
void      FLA_F2C( fla_random_tri_matrix_f  ) ( F_INT *uplo, F_INT* diag, F_INT *A, F_INT *IERROR );
void      FLA_F2C( fla_sqrt_f               ) ( F_INT *alpha, F_INT *IERROR );
void      FLA_F2C( fla_symmetrize_f         ) ( F_INT *uplo, F_INT *A, F_INT *IERROR );
void      FLA_F2C( fla_triangularize_f      ) ( F_INT *uplo, F_INT *diag, F_INT *A, F_INT *IERROR );
void      FLA_F2C( fla_transpose_f          ) ( F_INT *A, F_INT *IERROR );

// -----------------------------------------------------------------------------

FLA_Error FLA_Absolute_square_check( FLA_Obj alpha );
FLA_Error FLA_Conjugate_check( FLA_Obj A );
FLA_Error FLA_Conjugate_r_check( FLA_Uplo uplo, FLA_Obj A );
FLA_Error FLA_Hermitianize_check( FLA_Uplo uplo, FLA_Obj A );
FLA_Error FLA_Invert_check( FLA_Obj alpha );
FLA_Error FLA_Max_abs_value_check( FLA_Obj A, FLA_Obj amax );
FLA_Error FLA_Max_elemwise_diff_check( FLA_Obj A, FLA_Obj B );
FLA_Error FLA_Mult_add_check( FLA_Obj alpha, FLA_Obj beta, FLA_Obj gamma );
FLA_Error FLA_Negate_check( FLA_Obj x );
FLA_Error FLA_Norm1_check( FLA_Obj A, FLA_Obj norm );
FLA_Error FLA_Norm_inf_check( FLA_Obj A, FLA_Obj norm );
FLA_Error FLA_Random_matrix_check( FLA_Obj A );
FLA_Error FLA_Random_herm_matrix_check( FLA_Uplo uplo, FLA_Obj A );
FLA_Error FLA_Random_spd_matrix_check( FLA_Uplo uplo, FLA_Obj A );
FLA_Error FLA_Random_tri_matrix_check( FLA_Uplo uplo, FLA_Diag diag, FLA_Obj A );
FLA_Error FLA_Sqrt_check( FLA_Obj alpha );
FLA_Error FLA_Symmetrize_check( FLA_Uplo uplo, FLA_Obj A );
FLA_Error FLA_Triangularize_check( FLA_Uplo uplo, FLA_Diag diag, FLA_Obj A );
FLA_Error FLA_Transpose_check( FLA_Obj A );

// -----------------------------------------------------------------------------

FLA_Error FLA_Transpose_blk_var1( FLA_Obj A, fla_tpose_t* cntl );
FLA_Error FLA_Transpose_blk_var2( FLA_Obj A, fla_tpose_t* cntl );
FLA_Error FLA_Transpose_unb_var1( FLA_Obj A );
FLA_Error FLA_Transpose_unb_var2( FLA_Obj A );
FLA_Error FLA_Swap_t_blk_var1( FLA_Obj A, FLA_Obj B, fla_swap_t* cntl );
FLA_Error FLA_Swap_t_blk_var2( FLA_Obj A, FLA_Obj B, fla_swap_t* cntl );

