/*
   libflame
   An object-based infrastructure for developing high-performance
   dense linear algebra libraries.

   Copyright (C) 2009, The University of Texas

   libflame is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as
   published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   libflame is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with libflame; if you did not receive a copy, see
   http://www.gnu.org/licenses/.

   For more information, please contact us at flame@cs.utexas.edu or
   send mail to:

   Field G. Van Zee and/or
   Robert A. van de Geijn
   The University of Texas at Austin
   Department of Computer Sciences
   1 University Station C0500
   Austin TX 78712
*/

#include "FLA_util_lapack_prototypes_opt.h"

// --- LAPACK-related utility prototypes ---------------------------------------

FLA_Error FLA_Apply_pivots( FLA_Side side, FLA_Trans trans, FLA_Obj ipiv, FLA_Obj A );
FLA_Error FLA_Apply_househ2_UT( FLA_Obj tau, FLA_Obj u2, FLA_Obj a1t, FLA_Obj A2 );
FLA_Error FLA_Househ2_UT( FLA_Obj chi_1, FLA_Obj x2, FLA_Obj tau );
FLA_Error FLA_Form_perm_matrix( FLA_Obj ipiv, FLA_Obj A );
FLA_Error FLA_Shift_pivots_to( FLA_Pivot_type ptype, FLA_Obj ipiv );

void FLA_F2C( fla_apply_pivots_f            )( F_INT *side, F_INT *trans, F_INT *ipiv, F_INT *A, F_INT *IERROR );
void FLA_F2C( fla_form_perm_matrix_f        )( F_INT *ipiv, F_INT *A, F_INT *IERROR );
void FLA_F2C( fla_shift_pivots_to_f         )( F_INT *ptype, F_INT *ipiv, F_INT *IERROR );

// --- LAPACK-related utility check routine prototypes -------------------------

FLA_Error FLA_Apply_househ2_UT_check( FLA_Obj tau, FLA_Obj u2, FLA_Obj a1t, FLA_Obj A2 );
FLA_Error FLA_Shift_pivots_to_check( FLA_Pivot_type ptype, FLA_Obj ipiv );
FLA_Error FLA_Form_perm_matrix_check( FLA_Obj ipiv, FLA_Obj A );

